<?php

namespace Drupal\image_style_desc\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\image\Entity\ImageStyle;
use Drupal\image\Plugin\Field\FieldWidget\ImageWidget;

/**
 * Provides functionality for image style descriptions.
 */
class ImageStyleDescHelper {
  use StringTranslationTrait;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Constructs the service.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configuration factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(ConfigFactoryInterface $configFactory, EntityTypeManagerInterface $entityTypeManager) {
    $this->configFactory = $configFactory;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Alters image widget forms to add dynamic descriptions.
   *
   * @param array &$element
   *   The form element to alter.
   * @param array $context
   *   The widget context.
   */
  public function alterImageWidgetForm(array &$element, array $context): void {
    if (!$this->isImageWidget($context)) {
      return;
    }

    if ($this->isExcludedContentType($context)) {
      return;
    }

    $styles = $this->getImageStylesFromContext($context);
    if (empty($styles)) {
      return;
    }

    $this->addDescription($element, $styles);
  }

  /**
   * Checks if the widget is an ImageWidget.
   */
  private function isImageWidget(array $context): bool {
    return isset($context['widget']) && $context['widget'] instanceof ImageWidget;
  }

  /**
   * Checks if the content type is excluded from processing.
   */
  private function isExcludedContentType(array $context): bool {
    $excludedContentTypes = $this->configFactory->get('image_style_desc.settings')->get('excluded_content_types') ?? [];
    $fieldDefinition = $context['items']->getFieldDefinition();
    $bundle = $fieldDefinition->getTargetBundle();

    return in_array($bundle, $excludedContentTypes, TRUE);
  }

  /**
   * Retrieves image styles associated with the widget across all view modes.
   */
  private function getImageStylesFromContext(array $context): array {
    $fieldDefinition = $context['items']->getFieldDefinition();
    $applyToAllViewModes = $this->configFactory->get('image_style_desc.settings')->get('all_view_modes') ?? FALSE;

    $viewModes = $applyToAllViewModes
      ? $this->getAllViewModes($fieldDefinition->getTargetEntityTypeId())
      : ['default'];

    $styles = [];
    foreach ($viewModes as $viewModeId) {
      $viewDisplay = $this->entityTypeManager->getStorage('entity_view_display')->load(sprintf(
        '%s.%s.%s',
        $fieldDefinition->getTargetEntityTypeId(),
        $fieldDefinition->getTargetBundle(),
        $viewModeId
      ));

      if (!$viewDisplay) {
        continue;
      }

      $formatterSettings = $viewDisplay->getComponent($fieldDefinition->getName())['settings'] ?? NULL;
      if (!empty($formatterSettings['image_style'])) {
        $imageStyle = $this->entityTypeManager->getStorage('image_style')->load($formatterSettings['image_style']);
        if ($imageStyle) {
          $dimensions = $this->extractDimensions($imageStyle);
          $styles[] = [
            'label' => $imageStyle->label(),
            'dimensions' => $dimensions,
          ];
        }
      }
    }

    return $styles;
  }

  /**
   * Retrieves all view modes for a given entity type.
   */
  private function getAllViewModes(string $entityTypeId): array {
    $viewModes = $this->entityTypeManager
      ->getStorage('entity_view_mode')
      ->loadByProperties(['targetEntityType' => $entityTypeId]);

    $modes = array_map(
      fn($key) => str_replace("$entityTypeId.", '', $key),
      array_keys($viewModes)
    );

    if (!in_array('default', $modes)) {
      array_unshift($modes, 'default');
    }

    return $modes;
  }

  /**
   * Extracts dimensions from the image style effects.
   */
  private function extractDimensions(ImageStyle $imageStyle): string {
    foreach ($imageStyle->getEffects() as $effect) {
      $config = $effect->getConfiguration();
      if (!empty($config['data']['width']) || !empty($config['data']['height'])) {
        $width = $config['data']['width'] ?? 'auto';
        $height = $config['data']['height'] ?? 'auto';
        return "{$width}x{$height}";
      }
    }
    return 'unknown';
  }

  /**
   * Adds dynamic descriptions for multiple image styles to the widget element.
   */
  private function addDescription(array &$element, array $styles): void {
    if (!isset($element['widget'][0])) {
      return;
    }

    $currentDescription = $element['widget'][0]['#description'] ?? '';
    $descriptionTemplate = $this->configFactory->get('image_style_desc.settings')->get('image_style_desc')
      ?? 'The standard dimensions for the @style image style are @dimensions.';

    $descriptions = [];
    foreach ($styles as $style) {
      $descriptions[] = strtr($descriptionTemplate, [
        '@style' => $style['label'],
        '@dimensions' => $style['dimensions'],
      ]);
    }

    $descriptions = array_unique($descriptions);
    $element['widget'][0]['#description'] = trim($currentDescription . '<br>' . implode('<br>', $descriptions));
  }

}
